/*
	G. Wiesenekker. ZZZZZZ a chess program.
	Copyright (C) 1993  G. Wiesenekker
	E-mail: wiesenecker@sara.nl

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "zzzzzz.h"

#define START_BOARD "brd.txt"

/*
read_game reads the moves played sofar and returns the number of moves
done by white and by black.
*/

void read_game(FILE *f, int *nwhite, int *nblack)
{
	char line[LINE_MAX];
	char move[LINE_MAX];
	int ipv;
	
	*nwhite = *nblack = 0;
	while(fgets(line, LINE_MAX, f) != NULL)
	{
		if (sscanf(line, "%s", move) != 1) continue;
		if (*nwhite == *nblack)
			gen_pv(WHITE, BLACK);
		else
			gen_pv(BLACK, WHITE);
		BUG((ipv = search_pv(move, TRUE)) == INVALID)
		if (*nwhite == *nblack)
		{
			do_move(WHITE, BLACK, pv[ipv].pv_move);
			(*nwhite)++;
		}
		else
		{
			do_move(BLACK, WHITE, pv[ipv].pv_move);
			(*nblack)++;
		}
	}
	compact_lists();
}

/*
analyze_pos reads the positions from the file name, and analyzes them.
*/

void analyze_pos(char *name)
{
	FILE *f;
	int ipv;
	
	BUG((f = fopen(name, "r")) == NULL)
	while(read_board(f))
	{
		my_printf("pos board\n");
		display_board();
		display_list();
		display_occup();
		display_score();
		display_attack();
		gen_pv(WHITE, BLACK);
		my_printf("white moves:\n");
		for (ipv = 0; ipv < npv; ipv++)
		{
			do_move(WHITE, BLACK, pv[ipv].pv_move);
			undo_move(WHITE, BLACK);
			my_printf(" %5s", pv[ipv].pv_string);
			if (((ipv % 10) == 9) or (ipv == (npv - 1))) my_printf("\n");
		}
		my_printf("black moves:\n");
		gen_pv(BLACK, WHITE);
		for (ipv = 0; ipv < npv; ipv++)
		{
			do_move(BLACK, WHITE, pv[ipv].pv_move);
			undo_move(BLACK, WHITE);
			my_printf(" %5s", pv[ipv].pv_string);
			if (((ipv % 10) == 9) or (ipv == (npv - 1))) my_printf("\n");
		}
	}
	BUG(fclose(f) == EOF)
}

/*
analyze_game reads the move from the file name, and analyzes the positions
after each move.
*/

void analyze_game(char *name)
{
	FILE *f;
	int nwhite, nblack;
	char line[LINE_MAX];
	char move[LINE_MAX];
	int ipv;
	
	BUG((f = fopen(START_BOARD, "r")) == NULL)
	BUG(!read_board(f))
	BUG(fclose(f) == EOF)
	display_board();
	display_score();
	
	BUG((f = fopen(name, "r")) == NULL)
	nwhite = nblack = 0;
	while(fgets(line, LINE_MAX, f) != NULL)
	{
		if (sscanf(line, "%s", move) != 1) continue;
		if (nwhite == nblack)
			gen_pv(WHITE, BLACK);
		else
			gen_pv(BLACK, WHITE);
		BUG((ipv = search_pv(move, TRUE)) == INVALID)
		if (nwhite == nblack)
		{
			do_move(WHITE, BLACK, pv[ipv].pv_move);
			nwhite++;
		}
		else
		{
			do_move(BLACK, WHITE, pv[ipv].pv_move);
			nblack++;
		}
		display_board();
		display_score();
	}
	BUG(fclose(f) == EOF)
}

/*
solve_problems reads the problems from file name, and tries to solve them.
*/

double solve_problems(char *name)
{
	FILE *f;
	int nproblems, nsolved, nsimple, ninvalid;
	char col_to_move[LINE_MAX], move[LINE_MAX];
	int ipv;
	
	nproblems = nsolved = nsimple = ninvalid = 0;
	BUG((f = fopen(name, "r")) == NULL)
	while(read_board(f))
	{
		BUG(fscanf(f, "%s%s", col_to_move, move) != 2)
		nproblems++;
		display_board();
		display_list();
		display_occup();
		
		node->node_total_key = node->node_key;
		if (*col_to_move == 'W')
		{
			update_key(WHITE, &(node->node_total_key));
			gen_pv(WHITE, BLACK);
		}
		else
		{
			update_key(BLACK, &(node->node_total_key));
			gen_pv(BLACK, WHITE);
		}
		if (npv == 0)
		{
			my_printf("problem %d is draw or mate!\n", nproblems);
			nsimple++;
			continue;
		}
		if (npv == 1)
		{
			my_printf("problem %d has a forced move!\n", nproblems);
			nsimple++;
			continue;
		}
		ipv = search_pv(move, TRUE);
		if (ipv == INVALID)
		{
			my_printf("problem %d has an invalid move!\n", nproblems);
			ninvalid++;
			continue;
		}
		if (*col_to_move == 'W')
			do_pv(WHITE, BLACK);
		else
			do_pv(BLACK, WHITE);
		if (ipv == jpv)
		{
			my_printf("solved problem %d!\n", nproblems);
			nsolved++;
		}
		else
			my_printf("did not solve problem %d!\n", nproblems);
	}
	my_printf("nproblems=%d nsolved=%d nsimple=%d ninvalid=%d\n",
		nproblems, nsolved, nsimple, ninvalid);
	BUG(fclose(f) == EOF)
	if (nproblems == 0)
		return((double) 0);
	else
		return((double) nsolved / (double) nproblems);
}

/*
play_batch reads the moves played sofar from the file name, and appends 
a reply to the file name.
*/

void play_batch(char *name)
{
	int my_colour, your_colour;
	FILE *f;
	int nwhite, nblack;
	
	BUG((f = fopen(START_BOARD, "r")) == NULL)
	BUG(!read_board(f))
	BUG(fclose(f) == EOF)
	/*
	my_printf("start board\n");
	display_board();
	display_list();
	display_occup();
	display_score();
	*/
	
	BUG((f = fopen(name, "r")) == NULL)
	read_game(f, &nwhite, &nblack);
	BUG(fclose(f) == EOF)
	my_printf("board after read game\n");
	display_board();
	display_list();
	display_occup();
	display_score();
	
	if (draw_by_repetition(TRUE))
	{
		my_printf("draw by rep!\n");
		exit(EXIT_FAILURE);
	}
	
	if (nwhite == nblack)
	{
		my_colour = WHITE; your_colour = BLACK;
	}
	else
	{
		my_colour = BLACK; your_colour = WHITE;
	}
	gen_pv(my_colour, your_colour);
	if (npv == 0)
	{
		if (i_am_check(my_colour, your_colour))
			my_printf("%sis mate!\n", colour_as_string[my_colour]);
		else
			my_printf("stale mate!\n");
		exit(EXIT_FAILURE);
	}
	if (npv == 1)
	{
		my_printf("forced!\n");
		jpv = 0;
	}
	else
		do_pv(my_colour, your_colour);
	if (Append)
	{
		BUG((f = fopen(name, "a")) == NULL)
		fprintf(f, "%s zzzzzz %d\n", pv[jpv].pv_string, pv[jpv].pv_score);
		BUG(fclose(f) == EOF)
	}
}

#if MACHINE == MAC
#define GAME_TIME1 (10 * 60)
#define GAME_TIME2 (5 * 60)
#define NGAME_MOVES1 20
#define NGAME_MOVES2 40
#define GAME_MARGIN (1 * 60)
#else
#define GAME_TIME1 (90 * 60)
#define GAME_TIME2 (60 * 60)
#define NGAME_MOVES1 60
#define NGAME_MOVES2 40
#define GAME_MARGIN (5 * 60)
#endif

/*
play_game plays a game under tournamen conditions.
zzzzzz plays xxxxxx, you play yyyyyy.
*/

void play_game(int xxxxxx, int yyyyyy, char *name)
{
	FILE *f;
	int my_colour, your_colour;
	int nwhite, nblack;
	int ngame_moves;
	time_t wall1, wall2;
	int wall;
	int ipv;
	
	while(TRUE)
	{
		BUG((f = fopen(START_BOARD, "r")) == NULL)
		BUG(!read_board(f))
		BUG(fclose(f) == EOF)
		/*
		my_printf("start board\n");
		display_board();
		display_list();
		display_occup();
		display_score();
		*/
		
		BUG((f = fopen(name, "r")) == NULL)
		read_game(f, &nwhite, &nblack);
		BUG(fclose(f) == EOF)
		my_printf("board after read game\n");
		display_board();
		/*
		display_list();
		display_occup();
		display_score();
		*/
	
		if (draw_by_repetition(TRUE))
		{
			my_printf("draw by rep!\n");
			exit(EXIT_FAILURE);
		}
		
		if (nwhite == nblack)
		{
			my_colour = WHITE;
			your_colour = BLACK;
			ngame_moves = nwhite;
		}
		else
		{
			my_colour = BLACK;
			your_colour = WHITE;
			ngame_moves = nblack;
		}
		gen_pv(my_colour, your_colour);
		if (npv == 0)
		{
			if (i_am_check(my_colour, your_colour))
				my_printf("%sis mate!\n", colour_as_string[my_colour]);
			else
				my_printf("stale mate!\n");
			exit(EXIT_FAILURE);
		}
		else
		{
			if (xxxxxx == my_colour)
			{
				if (npv == 1)
				{
					my_printf("forced!\n");
					jpv = 0;
					pv[jpv].pv_score = 0;
				}
				else
				{
					/* calculate time limit */
					
					if (ngame_moves < NGAME_MOVES1)
					{
						TimeLimit =
						 	(GAME_TIME1 - GAME_MARGIN - GameTimeUsed) /
						 	(NGAME_MOVES1 - ngame_moves);
					}
					else if (ngame_moves < (NGAME_MOVES1 + NGAME_MOVES2))
					{
						TimeLimit =
							(GAME_TIME1 + GAME_TIME2 - GAME_MARGIN - GameTimeUsed) /
						  (NGAME_MOVES1 + NGAME_MOVES2 - ngame_moves);
					}
					else
					{
						TimeLimit = 90;
					}
					if (TimeLimit < 0)
					{
						my_printf("warning: negative time limit!\n");
						TimeLimit = 1;
					}
					my_printf("TimeLimit=%d\n", TimeLimit);
					wall1 = time(NULL);
					do_pv(my_colour, your_colour);
					wall2 = time(NULL);
					wall = (int) difftime(wall2, wall1) + 1;
					GameTimeUsed += wall;
					my_printf("WallTime=%d GameTimeUsed=%d\n", wall, GameTimeUsed);
				}
				my_printf("\azzzzzz move: %s\n", pv[jpv].pv_string);
				BUG((f = fopen(name, "a")) == NULL)
				fprintf(f, "%s zzzzzz %d\n", pv[jpv].pv_string, pv[jpv].pv_score);
				BUG(fclose(f) == EOF)
			}
			else
			{
				char line[LINE_MAX], move[LINE_MAX];
				
				if ((ngame_moves > 0) and (ngame_moves % 5) == 0)
				{
					while(TRUE)
					{
						my_printf("\aGameTimeUsed? (min) ");
						if (gets(line) == NULL) exit(EXIT_FAILURE);
						if (sscanf(line, "%d", &GameTimeUsed) != 1) continue;
						GameTimeUsed *= 60;
						break;
					}
				}
				/*
				Just for information.
				*/
				get_book_move(my_colour, your_colour);
				while(TRUE)
				{
					my_printf("your move? ");
					if (gets(line) == NULL) exit(EXIT_FAILURE);
					if (sscanf(line, "%s", move) != 1) continue;
					if (strcmp(move, "quit") == 0) exit(EXIT_SUCCESS);
					if (strcmp(move, "?") == 0)
					{
						for (ipv = 0; ipv < npv; ipv++)
						{
							my_printf(" %5s", pv[ipv].pv_string);
							if (((ipv % 10) == 9) or (ipv == (npv - 1))) my_printf("\n");
						}
						continue;
					}
					ipv = search_pv(move, FALSE);
					if (ipv == INVALID) continue;
					BUG((f = fopen(name, "a")) == NULL)
					fprintf(f, "%s\n", pv[ipv].pv_string);
					BUG(fclose(f) == EOF)
					break;
				}
			}
		}
	}
}

/*
do_what is used by the driver program par. It writes the legal moves
in the starting position to the file dbase.
*/

void do_what(char *name, char *dbase)
{
	FILE *f;
	int nwhite, nblack;
	int my_colour, your_colour;
	int ngame_moves;
	int ipv;
	
	BUG((f = fopen(START_BOARD, "r")) == NULL)
	BUG(!read_board(f))
	BUG(fclose(f) == EOF)
	
	BUG((f = fopen(name, "r")) == NULL)
	read_game(f, &nwhite, &nblack);
	BUG(fclose(f) == EOF)
	my_printf("board after read game\n");
	display_board();
	display_list();
	display_occup();

	BUG((f = fopen(dbase, "w")) == NULL)
	if (nwhite == nblack)
	{
		my_colour = WHITE;
		your_colour = BLACK;
		ngame_moves = nwhite;
	}
	else
	{
		my_colour = BLACK;
		your_colour = WHITE;
		ngame_moves = nblack;
	}
	fprintf(f, "%s%d\n", colour_as_string[my_colour], ngame_moves);
	if (draw_by_repetition(TRUE))
	{
		my_printf("draw by rep!\n");
		fprintf(f, "0\ndraw by repetition");
	}
	else
	{
		gen_pv(my_colour, your_colour);
		if (npv == 0)
		{
			if (i_am_check(my_colour, your_colour))
			{
				my_printf("%sis mate!\n", colour_as_string[my_colour]);
				fprintf(f, "0\nmate!\n");
			}
			else
			{
				my_printf("stale mate!\n");
				fprintf(f, "0\nstale mate!\n");
			}
		}
		else if (npv == 1)
		{
			fprintf(f, "1\n%s %08lx\n", pv[0].pv_string, pv[0].pv_move);
		}
		else
		{
			ipv = get_book_move(my_colour, your_colour);
			/*
			a-week-before-the-tournament-hack.
			If we have a book move, we make the first move a book move and (npv < 0).
			*/
			if (ipv != INVALID)
			{
				pv[npv] = pv[0];
				pv[0] = pv[ipv];
				pv[ipv] = pv[npv];
				fprintf(f, "%d\n", -npv);
				for (ipv = 0; ipv < npv; ipv++)
					fprintf(f, "%s %08lx\n", pv[ipv].pv_string, pv[ipv].pv_move);
			}
			else
			{
				fprintf(f, "%d\n", npv);
				for (ipv = 0; ipv < npv; ipv++)
					fprintf(f, "%s %08lx\n", pv[ipv].pv_string, pv[ipv].pv_move);
			}
		}
	}
	BUG(fclose(f) == EOF)
}

/*
The moves to be analyzed are read from the file dbase, and 
the best move is written back to the file dbase
*/

void play_par(char *name, char *dbase)
{
	FILE *f;
	int nwhite, nblack;
	int my_colour, your_colour;
	int ipv;
	
	BUG((f = fopen(START_BOARD, "r")) == NULL)
	BUG(!read_board(f))
	BUG(fclose(f) == EOF)
	
	BUG((f = fopen(name, "r")) == NULL)
	read_game(f, &nwhite, &nblack);
	BUG(fclose(f) == EOF)
	my_printf("board after read game\n");
	display_board();
	display_list();
	display_occup();

	if (nwhite == nblack)
	{
		my_colour = WHITE;
		your_colour = BLACK;
	}
	else
	{
		my_colour = BLACK;
		your_colour = WHITE;
	}
	BUG((f = fopen(dbase, "r")) == NULL)
	BUG(fscanf(f, "%d", &npv) != 1)
	BUG(npv == 0)
	for (ipv = 0; ipv < npv; ipv++)
		BUG(fscanf(f, "%s%lx", pv[ipv].pv_string, &(pv[ipv].pv_move)) != 2)
	BUG(fclose(f) == EOF)
	do_pv(my_colour, your_colour);
	BUG((f = fopen(dbase, "w")) == NULL)
	fprintf(f, "%s %d\n", pv[jpv].pv_string, pv[jpv].pv_score);
	BUG(fclose(f) == EOF)
}

void gen_book(char *name)
{
#define NBOOK_MAX 4096
	FILE *f, *g;
	char move[LINE_MAX];
	int ipv;
	int my_colour, your_colour;
	int suspect, behind;
	int ngame, nsuspect;
	pv_t var[NPVS_MAX];
	int nvar;
	book_t *with;
	long ntrans[2];
	int ivar, icolour, ibook;
	
	MALLOC(book[WHITE], book_t, NBOOK_MAX)
	MALLOC(book[BLACK], book_t, NBOOK_MAX)
	BUG((f = fopen(START_BOARD, "r")) == NULL)
	BUG(!read_board(f))
	BUG(fclose(f) == EOF)
	BUG((f = fopen(name, "r")) == NULL)
	BUG((g = fopen("all.txt", "w")) == NULL)
	nvar = 0;
	suspect = FALSE;
	ngame = nsuspect = 0;
	nbook[WHITE] = nbook[BLACK] = 0;
	ntrans[WHITE] = ntrans[BLACK] = 0;
	while(fscanf(f, "%s", move) == 1)
	{
		if (*move == '!')
		{
			/* add variation */
			if (suspect)
			{
				my_printf("ignoring variation %d!\n", ngame);
				nsuspect++;
				for (ivar = nvar; ivar > 0; ivar--)
				{
					if (ivar & 1)
						undo_move(WHITE, BLACK);
					else
						undo_move(BLACK, WHITE);
				}
			}
			else
			{
				for (ivar = nvar; ivar > 0; ivar--)
				{
					if (ivar & 1)
					{
						my_colour = WHITE; your_colour = BLACK;
					}
					else
					{
						my_colour = BLACK; your_colour = WHITE;
					}
					for (ibook = 0; ibook < nbook[my_colour]; ibook++)
					{
						with = book[my_colour] + ibook;
						if ((with->book_tot_count[WHITE] == tot_count[WHITE]) and
						    (with->book_tot_count[BLACK] == tot_count[BLACK]) and
						    (with->book_total_key.hi == node->node_total_key.hi) and
						    (with->book_total_key.lo == node->node_total_key.lo)) break;
					}
					if (ibook < nbook[my_colour])
					{
						ntrans[my_colour]++;
						with->book_hit++;
						undo_move(my_colour, your_colour);
					}
					else
					{
						BUG(nbook[my_colour] >= NBOOK_MAX)
						if ((node->node_total_key.hi == 0x4948d2a5) and
						    (node->node_total_key.lo == 0x2c0fe4cd))
						{
							display_board();
							my_printf("ngame=%d\n", ngame);
						}
						with = book[my_colour] + nbook[my_colour];
						with->book_tot_count[WHITE] = tot_count[WHITE];
						with->book_tot_count[BLACK] = tot_count[BLACK];
						with->book_total_key = node->node_total_key;
						with->book_hit = 1;
						nbook[my_colour]++;
						undo_move(my_colour, your_colour);
						if (ivar == nvar)
						{
							write_board(g);
							if (my_colour == WHITE)
								fprintf(g, "W %s\n", var[ivar - 1].pv_string);
							else
								fprintf(g, "B %s\n", var[ivar - 1].pv_string);
						}
					}
				}
			}
			nvar = 0;
			suspect = FALSE;
			ngame++;
			continue;
		}
		if (nvar & 1)
		{
			my_colour = BLACK; your_colour = WHITE;
		}
		else
		{
			my_colour = WHITE; your_colour = BLACK;
		}
		/* judge position */
		if (node->node_mat[my_colour] < node->node_mat[your_colour])
			behind = TRUE;
		else
			behind = FALSE;
		gen_pv(my_colour, your_colour);
		BUG((ipv = search_pv(move, TRUE)) == INVALID)
		BUG(nvar >= NPVS_MAX)
		var[nvar] = pv[ipv];
		do_move(my_colour, your_colour, var[nvar].pv_move);
		nvar++;
		if (behind)
		{
			if (node->node_mat[my_colour] < node->node_mat[your_colour])
			{
				if (!suspect)
					suspect = TRUE;
			}
		}
	}
	BUG(fclose(f) == EOF)
	BUG(fclose(g) == EOF)
	for (icolour = WHITE; icolour <= BLACK; icolour++)
	{
		if (nbook[icolour] > 0)
		{
			BUG((f = fopen(book_name[icolour], "wb")) == NULL)
			BUG(fwrite(&(nbook[icolour]), sizeof(long), 1, f) != 1)
			BUG(fwrite(book[icolour], sizeof(book_t), nbook[icolour], f) != nbook[icolour])
			BUG(fclose(f) == EOF)
			my_printf("wrote %sbook size=%ld trans=%ld\n",
				colour_as_string[icolour], nbook[icolour], ntrans[icolour]);
		}
	}
	free(book[BLACK]);
	free(book[WHITE]);
	my_printf("ngame=%d nsuspect=%d\n", ngame, nsuspect);
}

#define NDIM 23
#define PROBLEMS_FILE "opt.txt"
#define INIT_FILE "init.txt"
#define RESULTS_FILE "results.txt"

void try(void);
void try(void)
{
	FILE *f, *g;
	double err;
	int i;
	double y[NDIM];
	
	BUG((f = fopen(INIT_FILE, "r")) == NULL)
	BUG((g = fopen(RESULTS_FILE, "a")) == NULL)
	while(fscanf(f, "%lf", &err) == 1)
	{
		BUG(err > 1.0)
		for (i = 0; i < NDIM; i++)
			BUG(fscanf(f, "%lf", y + i) != 1)
		if (err < 0)
		{
			set_score(y);
			err = solve_problems(PROBLEMS_FILE);
			fprintf(g, "%.2f", err);
			for (i = 0; i < NDIM; i++)
				fprintf(g, " %d", (int) y[i]);
			fprintf(g, "\n");
		}
	}
	BUG(fclose(g) == EOF)
	BUG(fclose(f) == EOF)
}

#if MACHINE == MAC
#define ARGC_MAX 256
int argc;
char *argv_table[ARGC_MAX];
char **argv = argv_table;
int main(void);
int main(void)
#else
int main(int argc, char **argv);
int main(int argc, char **argv)
#endif
{
#if MACHINE == MAC
	{
		FILE *f;
		char line[LINE_MAX];
	
		BUG((f = fopen("arg.txt", "r")) == NULL)
		argc = 0;
		while (fscanf(f, "%s", line) == 1)
		{
			BUG(argc >= ARGC_MAX)
			MALLOC(argv_table[argc], char, strlen(line) + 1)
			strcpy(argv_table[argc], line);
			argc++;
		}
		BUG(fclose(f) == EOF)
	}
#else
	argc--;
	argv++;
#endif

	my_printf("ZZZZZZ version 3.0, Copyright (C) 1993 G. Wiesenekker\n");
	my_printf("ZZZZZZ comes with ABSOLUTELY NO WARRANTY.\n");
	my_printf("ZZZZZZ is free software, and you are welcome to redistribute it\n");
	my_printf("under certain conditions.\n");
	my_printf("See the file COPYING (of which you should have received a copy\n");
	my_printf("along with ZZZZZZ) for details.\n");

	/* parse command line options */
	
	while((argc > 0) and (*argv[0] == '-'))
	{
		if (strcmp(argv[0], "-DepthMax") == 0)
		{
			BUG(argc < 2)
			BUG(sscanf(argv[1], "%d", &DepthMax) != 1)
			argc--; argc--;
			argv++; argv++;
		}
		else if (strcmp(argv[0], "-TimeLimit") == 0)
		{
			BUG(argc < 2)
			BUG(sscanf(argv[1], "%d", &TimeLimit) != 1)
			argc--; argc--;
			argv++; argv++;
		}
		else if (strcmp(argv[0], "-GameTimeUsed") == 0)
		{
			BUG(argc < 2)
			BUG(sscanf(argv[1], "%d", &GameTimeUsed) != 1)
			argc--; argc--;
			argv++; argv++;
		}
		else if (strcmp(argv[0], "-NoAppend") == 0)
		{
			Append = FALSE;
			argc--;
			argv++;
		}
		else if (strcmp(argv[0], "-NoBook") == 0)
		{
			UseBook = FALSE;
			argc--;
			argv++;
		}
		else
		{
			my_printf("unknown option %s ignored\n", argv[0]);
			argc--;
			argv++;
		}
	}
	
	my_printf("DepthMax=%d UseBook=%d TimeLimit=%d GameTimeUsed=%d\n",
		DepthMax, UseBook, TimeLimit, GameTimeUsed);
	
	init();
	
	if (strcmp(argv[0], "pos") == 0)
	{
		BUG(argc < 2)
		analyze_pos(argv[1]);
	}
	else if (strcmp(argv[0], "anal") == 0)
	{
		BUG(argc < 2)
		analyze_game(argv[1]);
	}
	else if (strcmp(argv[0], "solve") == 0)
	{
		BUG(argc < 2)
		solve_problems(argv[1]);
	}
	else if (strcmp(argv[0], "batch") == 0)
	{
		BUG(argc < 2)
		play_batch(argv[1]);
	}
	else if (strcmp(argv[0], "white") == 0)
	{
		BUG(argc < 2)
		play_game(WHITE, BLACK, argv[1]);
	}
	else if (strcmp(argv[0], "black") == 0)
	{
		BUG(argc < 2)
		play_game(BLACK, WHITE, argv[1]);
	}
	else if (strcmp(argv[0], "what") == 0)
	{
		BUG(argc < 3)
		do_what(argv[1], argv[2]);
	}
	else if (strcmp(argv[0], "par") == 0)
	{
		BUG(argc < 3)
		play_par(argv[1], argv[2]);
	}
	else if (strcmp(argv[0], "book") == 0)
	{
		BUG(argc < 2)
		gen_book(argv[1]);
	}
	else if (strcmp(argv[0], "try") == 0)
	{
		try();
	}
	else
	{
		my_printf("unknown arg %s!\n", argv[0]);
		exit(EXIT_FAILURE);
	}
	return(EXIT_SUCCESS);
}
